{
	base => 'value',
	number => 42,
	nested => {
		key => 'nested_value',
		deep => {
			level => 'deep_value',
		},
	},
	items => [qw(one two)],
	removed => [qw(keep remove)],
	replaced => [qw(old)],
}

